<?php

/**
 * 
 * 接口访问类，包含所有微信支付API列表的封装，类中方法为static方法，
 * 每个接口有默认超时时间（除提交被扫支付为10s，上报超时时间为1s外，其他均为6s）
 * @author widyhu
 *
 */
class WxPayApi
{
	/**
	 * 
	 * 统一下单，WxPayUnifiedOrder中out_trade_no、body、total_fee、trade_type必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayUnifiedOrder $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function unifiedOrder($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/pay/unifiedorder";

		if (!$inputObj->IsOut_trade_noSet()) {
			throw new WxPayException("缺少统一支付接口必填参数out_trade_no！");
		}
		else if (!$inputObj->IsBodySet()) {
			throw new WxPayException("缺少统一支付接口必填参数body！");
		}
		else if (!$inputObj->IsTotal_feeSet()) {
			throw new WxPayException("缺少统一支付接口必填参数total_fee！");
		}
		else if (!$inputObj->IsTrade_typeSet()) {
			throw new WxPayException("缺少统一支付接口必填参数trade_type！");
		}

		if (($inputObj->GetTrade_type() == "JSAPI") && !$inputObj->IsOpenidSet()) {
			throw new WxPayException("统一支付接口中，缺少必填参数openid！trade_type为JSAPI时，openid为必填参数！");
		}

		if (($inputObj->GetTrade_type() == "NATIVE") && !$inputObj->IsProduct_idSet()) {
			throw new WxPayException("统一支付接口中，缺少必填参数product_id！trade_type为JSAPI时，product_id为必填参数！");
		}

		if (!$inputObj->IsNotify_urlSet()) {
			$inputObj->SetNotify_url(WxPayConfig::NOTIFY_URL);
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetSpbill_create_ip($_SERVER["REMOTE_ADDR"]);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 查询订单，WxPayOrderQuery中out_trade_no、transaction_id至少填一个
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayOrderQuery $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function orderQuery($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/pay/orderquery";
		if (!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {
			throw new WxPayException("订单查询接口中，out_trade_no、transaction_id至少填一个！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 关闭订单，WxPayCloseOrder中out_trade_no必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayCloseOrder $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function closeOrder($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/pay/closeorder";

		if (!$inputObj->IsOut_trade_noSet()) {
			throw new WxPayException("订单查询接口中，out_trade_no必填！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 申请退款，WxPayRefund中out_trade_no、transaction_id至少填一个且
	 * out_refund_no、total_fee、refund_fee、op_user_id为必填参数
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayRefund $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function refund($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/secapi/pay/refund";
		if (!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {
			throw new WxPayException("退款申请接口中，out_trade_no、transaction_id至少填一个！");
		}
		else if (!$inputObj->IsOut_refund_noSet()) {
			throw new WxPayException("退款申请接口中，缺少必填参数out_refund_no！");
		}
		else if (!$inputObj->IsTotal_feeSet()) {
			throw new WxPayException("退款申请接口中，缺少必填参数total_fee！");
		}
		else if (!$inputObj->IsRefund_feeSet()) {
			throw new WxPayException("退款申请接口中，缺少必填参数refund_fee！");
		}
		else if (!$inputObj->IsOp_user_idSet()) {
			throw new WxPayException("退款申请接口中，缺少必填参数op_user_id！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, true, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 查询退款
	 * 提交退款申请后，通过调用该接口查询退款状态。退款有一定延时，
	 * 用零钱支付的退款20分钟内到账，银行卡支付的退款3个工作日后重新查询退款状态。
	 * WxPayRefundQuery中out_refund_no、out_trade_no、transaction_id、refund_id四个参数必填一个
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayRefundQuery $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function refundQuery($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/pay/refundquery";
		if (!$inputObj->IsOut_refund_noSet() && !$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet() && !$inputObj->IsRefund_idSet()) {
			throw new WxPayException("退款查询接口中，out_refund_no、out_trade_no、transaction_id、refund_id四个参数必填一个！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 下载对账单，WxPayDownloadBill中bill_date为必填参数
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayDownloadBill $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function downloadBill($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/pay/downloadbill";

		if (!$inputObj->IsBill_dateSet()) {
			throw new WxPayException("对账单接口中，缺少必填参数bill_date！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);

		if (substr($response, 0, 5) == "<xml>") {
			return "";
		}

		return $response;
	}

	/**
	 * 提交被扫支付API
	 * 收银员使用扫码设备读取微信用户刷卡授权码以后，二维码或条码信息传送至商户收银台，
	 * 由商户收银台或者商户后台调用该接口发起支付。
	 * WxPayWxPayMicroPay中body、out_trade_no、total_fee、auth_code参数必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayWxPayMicroPay $inputObj
	 * @param int $timeOut
	 */
	static public function micropay($inputObj, $timeOut = 10)
	{
		$url = "https://api.mch.weixin.qq.com/pay/micropay";

		if (!$inputObj->IsBodySet()) {
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数body！");
		}
		else if (!$inputObj->IsOut_trade_noSet()) {
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数out_trade_no！");
		}
		else if (!$inputObj->IsTotal_feeSet()) {
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数total_fee！");
		}
		else if (!$inputObj->IsAuth_codeSet()) {
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数auth_code！");
		}

		$inputObj->SetSpbill_create_ip($_SERVER["REMOTE_ADDR"]);
		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 撤销订单API接口，WxPayReverse中参数out_trade_no和transaction_id必须填写一个
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayReverse $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 */
	static public function reverse($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/secapi/pay/reverse";
		if (!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {
			throw new WxPayException("撤销订单API接口中，参数out_trade_no和transaction_id必须填写一个！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, true, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
	 * 
	 * 测速上报，该方法内部封装在report中，使用时请注意异常流程
	 * WxPayReport中interface_url、return_code、result_code、user_ip、execute_time_必填
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayReport $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function report($inputObj, $timeOut = 1)
	{
		$url = "https://api.mch.weixin.qq.com/payitil/report";

		if (!$inputObj->IsInterface_urlSet()) {
			throw new WxPayException("接口URL，缺少必填参数interface_url！");
		}

		if (!$inputObj->IsReturn_codeSet()) {
			throw new WxPayException("返回状态码，缺少必填参数return_code！");
		}

		if (!$inputObj->IsResult_codeSet()) {
			throw new WxPayException("业务结果，缺少必填参数result_code！");
		}

		if (!$inputObj->IsUser_ipSet()) {
			throw new WxPayException("访问接口IP，缺少必填参数user_ip！");
		}

		if (!$inputObj->IsExecute_time_Set()) {
			throw new WxPayException("接口耗时，缺少必填参数execute_time_！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetUser_ip($_SERVER["REMOTE_ADDR"]);
		$inputObj->SetTime(date("YmdHis"));
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		return $response;
	}

	/**
	 * 
	 * 生成二维码规则,模式一生成支付二维码
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayBizPayUrl $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function bizpayurl($inputObj, $timeOut = 6)
	{
		if (!$inputObj->IsProduct_idSet()) {
			throw new WxPayException("生成二维码，缺少必填参数product_id！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetTime_stamp(time());
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		return $inputObj->GetValues();
	}

	/**
	 * 
	 * 转换短链接
	 * 该接口主要用于扫码原生支付模式一中的二维码链接转成短链接(weixin://wxpay/s/XXXXXX)，
	 * 减小二维码数据量，提升扫描速度和精确度。
	 * appid、mchid、spbill_create_ip、nonce_str不需要填入
	 * @param WxPayShortUrl $inputObj
	 * @param int $timeOut
	 * @throws WxPayException
	 * @return 成功时返回，其他抛异常
	 */
	static public function shorturl($inputObj, $timeOut = 6)
	{
		$url = "https://api.mch.weixin.qq.com/tools/shorturl";

		if (!$inputObj->IsLong_urlSet()) {
			throw new WxPayException("需要转换的URL，签名用原串，传输需URL encode！");
		}

		$inputObj->SetAppid(WxPayConfig::APPID);
		$inputObj->SetMch_id(WxPayConfig::MCHID);
		$inputObj->SetNonce_str(self::getNonceStr());
		$inputObj->SetSign();
		$xml = $inputObj->ToXml();
		$startTimeStamp = self::getMillisecond();
		$response = self::postXmlCurl($xml, $url, false, $timeOut);
		$result = WxPayResults::Init($response);
		self::reportCostTime($url, $startTimeStamp, $result);
		return $result;
	}

	/**
 	 * 
 	 * 支付结果通用通知
 	 * @param function $callback
 	 * 直接回调函数使用方法: notify(you_function);
 	 * 回调类成员函数方法:notify(array($this, you_function));
 	 * $callback  原型为：function function_name($data){}
 	 */
	static public function notify($callback, &$msg)
	{
		$xml = $GLOBALS["HTTP_RAW_POST_DATA"];

		try {
			$result = WxPayResults::Init($xml);
		}
		catch (WxPayException $e) {
			$msg = $e->errorMessage();
			return false;
		}

		return call_user_func($callback, $result);
	}

	/**
	 * 
	 * 产生随机字符串，不长于32位
	 * @param int $length
	 * @return 产生的随机字符串
	 */
	static public function getNonceStr($length = 32)
	{
		$chars = "abcdefghijklmnopqrstuvwxyz0123456789";
		$str = "";

		for ($i = 0; $i < $length; $i++) {
			$str .= substr($chars, mt_rand(0, strlen($chars) - 1), 1);
		}

		return $str;
	}

	/**
	 * 直接输出xml
	 * @param string $xml
	 */
	static public function replyNotify($xml)
	{
		echo $xml;
	}

	/**
	 * 
	 * 上报数据， 上报的时候将屏蔽所有异常流程
	 * @param string $usrl
	 * @param int $startTimeStamp
	 * @param array $data
	 */
	static private function reportCostTime($url, $startTimeStamp, $data)
	{
		if (WxPayConfig::REPORT_LEVENL == 0) {
			return NULL;
		}

		if ((WxPayConfig::REPORT_LEVENL == 1) && array_key_exists("return_code", $data) && ($data["return_code"] == "SUCCESS") && array_key_exists("result_code", $data) && ($data["result_code"] == "SUCCESS")) {
			return NULL;
		}

		$endTimeStamp = self::getMillisecond();
		$objInput = new WxPayReport();
		$objInput->SetInterface_url($url);
		$objInput->SetExecute_time_($endTimeStamp - $startTimeStamp);

		if (array_key_exists("return_code", $data)) {
			$objInput->SetReturn_code($data["return_code"]);
		}

		if (array_key_exists("return_msg", $data)) {
			$objInput->SetReturn_msg($data["return_msg"]);
		}

		if (array_key_exists("result_code", $data)) {
			$objInput->SetResult_code($data["result_code"]);
		}

		if (array_key_exists("err_code", $data)) {
			$objInput->SetErr_code($data["err_code"]);
		}

		if (array_key_exists("err_code_des", $data)) {
			$objInput->SetErr_code_des($data["err_code_des"]);
		}

		if (array_key_exists("out_trade_no", $data)) {
			$objInput->SetOut_trade_no($data["out_trade_no"]);
		}

		if (array_key_exists("device_info", $data)) {
			$objInput->SetDevice_info($data["device_info"]);
		}

		try {
			self::report($objInput);
		}
		catch (WxPayException $e) {
		}
	}

	/**
	 * 以post方式提交xml到对应的接口url
	 * 
	 * @param string $xml  需要post的xml数据
	 * @param string $url  url
	 * @param bool $useCert 是否需要证书，默认不需要
	 * @param int $second   url执行超时时间，默认30s
	 * @throws WxPayException
	 */
	static private function postXmlCurl($xml, $url, $useCert = false, $second = 30)
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_TIMEOUT, $second);
		if ((WxPayConfig::CURL_PROXY_HOST != "0.0.0.0") && (WxPayConfig::CURL_PROXY_PORT != 0)) {
			curl_setopt($ch, CURLOPT_PROXY, WxPayConfig::CURL_PROXY_HOST);
			curl_setopt($ch, CURLOPT_PROXYPORT, WxPayConfig::CURL_PROXY_PORT);
		}

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		if ($useCert == true) {
			curl_setopt($ch, CURLOPT_SSLCERTTYPE, "PEM");
			curl_setopt($ch, CURLOPT_SSLCERT, WxPayConfig::SSLCERT_PATH);
			curl_setopt($ch, CURLOPT_SSLKEYTYPE, "PEM");
			curl_setopt($ch, CURLOPT_SSLKEY, WxPayConfig::SSLKEY_PATH);
		}

		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
		$data = curl_exec($ch);

		if ($data) {
			curl_close($ch);
			return $data;
		}
		else {
			$error = curl_errno($ch);
			curl_close($ch);
			throw new WxPayException("curl出错，错误码:$error");
		}
	}

	/**
	 * 获取毫秒级别的时间戳
	 */
	static private function getMillisecond()
	{
		$time = explode(" ", microtime());
		$time = $time[1] . ($time[0] * 1000);
		$time2 = explode(".", $time);
		$time = $time2[0];
		return $time;
	}
}

require_once ("WxPay.Exception.php");
require_once ("WxPay.Config.php");
require_once ("WxPay.Data.php");

?>
